//
// C++ Implementation: powermanager
//
// Author: Oliver Groß <z.o.gross@gmx.de>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
#include <QtGui>
#include "powermanager.h"
#include "common.h"

#include "batteryicon.h"
#define TIME_VALUE 3000

namespace qbat {
using namespace std;

CPowerManager::CPowerManager(QObject * parent) :
    QObject(parent),
    m_Timer(-1),
    m_DataTimer(-1),

    m_ACPlug(false),
    m_CriticalHandled(false)
{


    DefaultIcon = new CBatteryIcon(/*&m_Settings, */tr("merged"));

    if (CBatteryIcon::sysfsDir.exists()) {
        m_DataTimer = startTimer(15000);
        m_Timer = startTimer(/*m_Settings.pollingRate*/ TIME_VALUE);
    }
    updateSupplies();
}

CPowerManager::~CPowerManager() {
    killTimer(m_Timer);
    killTimer(m_DataTimer);
}


void CPowerManager::timerEvent(QTimerEvent * event) {
    if (event->timerId() == m_Timer)
        updateSupplies();
    else if (event->timerId() == m_DataTimer)
        updateBatteryData();
}

void CPowerManager::updateSupplies() {
    if (CBatteryIcon::sysfsDir.exists()) {
        //qDebug()<<"updateSupplies";
        bool oldAC = m_ACPlug;
        m_ACPlug = false;

        QStringList powerSupplies = CBatteryIcon::sysfsDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);

        QList<CBatteryIcon *> newBatteryIcons;
        CBatteryIcon * currentBatteryIcon;

        foreach(QString i, powerSupplies) {

            if (CBatteryIcon::sysfsDir.exists(i + "/online")) {
                if (readIntSysFile(CBatteryIcon::sysfsDir.filePath(i + "/online").toAscii().constData()) == 1) {
                    m_ACPlug = true;
                    m_CriticalHandled = false;
                }
            }else {

                if (m_BatteryIcons.contains(i)) {

                    currentBatteryIcon = m_BatteryIcons.take(i);
                } else {


                    currentBatteryIcon = new CBatteryIcon(/*&m_Settings,*/ i);

                    currentBatteryIcon->updateData();


                    m_CriticalHandled = false;
                }
                newBatteryIcons << currentBatteryIcon;
            }
        }

        foreach(CBatteryIcon * i, m_BatteryIcons)
            delete i;

        m_BatteryIcons.clear();

        foreach(CBatteryIcon * i, newBatteryIcons)
            m_BatteryIcons.insert(i->data().name, i);

        if (oldAC != m_ACPlug)
            updateBatteryData();
        else
            updateMergedData();

        if (m_BatteryIcons.isEmpty()) {
            //TODO:Add icon battery empty
            //if battery is
        }
        else {

            DefaultIcon->updateData();

        }

        checkCritical();
    }
    else {
        //DefaultIcon->setMessage(tr("no information available"));

    }

}

void CPowerManager::updateMergedData() {
    if (!m_BatteryIcons.isEmpty()) {
        // qDebug()<<"updateMergedData";
        int fullCount = 0;
        int rateCount = 0;
        int voltageCount = 0;

        int fullCapacity    = 0;
        int currentCapacity = 0;
        int rate            = 0;
        int voltage         = 0;
        int status          = 0;

        foreach (CBatteryIcon * battery, m_BatteryIcons)
        {
            if (battery->data().voltage) {
                voltage += battery->data().voltage;
                voltageCount++;
            }

            if (!battery->data().energyUnits) {
                currentCapacity += battery->data().currentCapacity;

                if (battery->data().fullCapacity) {
                    fullCapacity += battery->data().fullCapacity;
                    fullCount++;
                }

                if (battery->data().rate) {
                    rate += battery->data().rate;
                    rateCount++;
                }
            }
            else if (battery->data().voltage) {
                double voltageNorm  = battery->data().voltage / 100.0;
                currentCapacity += qRound(battery->data().currentCapacity / voltageNorm);

                if (battery->data().fullCapacity) {
                    fullCapacity += qRound(battery->data().fullCapacity / voltageNorm);
                    fullCount++;
                }

                if (battery->data().rate) {
                    rate += qRound(battery->data().rate / voltageNorm);
                    rateCount++;
                }
            }

            if ((!status) || (battery->data().status != UI_BATTERY_FULL))
                status = battery->data().status;
        }

        if (fullCount)
            fullCapacity /= fullCount;

        if (rateCount)
            rate /= rateCount;

        if (voltageCount)
            voltage /= voltageCount;

        currentCapacity /= m_BatteryIcons.count();

        DefaultIcon->updateData(currentCapacity, fullCapacity, 0, rate, voltage, status, false);
        emit batteryChanged();
    }
}

void CPowerManager::checkCritical() {
    if (m_ACPlug || m_CriticalHandled ||DefaultIcon->data().relativeCharge == -1
            /*|| !m_Settings.handleCritical ||
                                    DefaultIcon->data().relativeCharge > m_Settings.criticalCapacity*/)
        return;

    //  QString msgTitle = (m_Settings.executeCommand && m_Settings.confirmWithTimeout) ?
    //     tr("QBat - critical battery capacity (will automatically choose ok on timeout)"):
    //    tr("QBat - critical battery capacity");
    //  QString msgText = (m_Settings.executeCommand && m_Settings.confirmCommand) ?
    //    tr("WARNING: The attached battery(s) reached the critical mark.\n"
    //      "Click cancel and please make sure to save and shut down soon or provide another source of power\n"
    //      "or:\n"
    //     "Click ok to execute:\n%1").arg(m_Settings.criticalCommand) :
    //  tr("WARNING: The attached battery(s) reached the critical mark.\n"
    //     "Please make sure to save and shut down soon or provide another source of power.");
    //    if (m_Settings.executeCommand && (!m_Settings.confirmCommand || (QTimerMessageBox::warning(NULL, msgTitle, msgText,
    //                                                                                               (m_Settings.confirmWithTimeout) ? m_Settings.timeoutValue : -1, QMessageBox::Ok | QMessageBox::Cancel))))
    //        QProcess::startDetached(m_Settings.criticalCommand);
    //    else {
    //        if (m_Settings.showBalloon)
    //            m_BatteryIcons.begin().value()->showMessage(msgTitle, msgText, QSystemTrayIcon::Warning, 7000);
    //        else
    //            QMessageBox::warning(NULL, msgTitle, msgText);
    //    }
    m_CriticalHandled = true;
}

void CPowerManager::updateBatteryData() {
  //  qDebug()<<"updateBatteryData";
    foreach (CBatteryIcon * battery, m_BatteryIcons)
        battery->updateData();

    updateMergedData();
    checkCritical();
}

void CPowerManager::restartTimer() {
    killTimer(m_Timer);
    m_Timer = startTimer(/*m_Settings.pollingRate*/ TIME_VALUE);
    killTimer(m_DataTimer);
    m_DataTimer = startTimer(15000);
}



}
